# Lotka volterra model:

# Y1 --> 2*Y1, bith of prey with parameter c1
# Y1 + Y2 -->  2*Y2, % birth of predator, depends on prey, parameter c2
# Y2 --> 0, death of prey with parameter c3

IC=c(50, 100) # initial conditions
c1=1
c2=0.005
c3= 0.6
th=c(c1,c2,c3) # parameters (rate constants)

h=function(y, th){return(c(th[1]*y[1], th[2]*y[1]*y[2], th[3]*y[2] ))}

n=10000 # number of realizations

gillespieLotkaVolterra <- function(x, th,h, n)
  # x initial conditions vector
  # th: parameter vector 
  # h: kinetic equations vector (hazards)
  # n number of realizations
  
{
  u=length(x)
  v=length(th) # number of reactions (IN THIS CASE: same as number of kinetic parameters)
  tt=0 # initialize time
  tvec=vector("numeric",n) # create a vector for the time, n number of realizations
  xmat=matrix(0, ncol=u, nrow=n+1) # matrix for the u variables of the system
  xmat[1,]=x # initial state
  for (i in 1:n){
    totH= sum(h(x, th))# total hazard
    tt=tt+rexp(1,totH) # sample time to next event from exp probability distribution
    j=sample(v,1, prob=h(x, th)) # choose a reaction to occur
    
    # update state vector
    if ( j==1) {
      x[1]=x[1]+1 # a prey is born, nothing happens to the predator
    } else if (j==2) {
      x[2]=x[2]+1 # a predator is born, and a prey is eaten up :(
      x[1]=x[1]-1
    } else # j==3
      x[2]=x[2]-1 # a predator dies, nothing happens to the prey
    
    # done, just fill in the vectors:
    tvec[i]=tt
    xmat[i+1, ]=x
  } # end for loop
  return(list(t=tvec, x=xmat))
} # end function
    

out=gillespieLotkaVolterra(IC, th,h, n)
  

#op=par(mfrow=c(2,2))
plot(stepfun(out$t, out$x[,1]), pch="")
plot(stepfun(out$t, out$x[,2]), pch="")
plot(out$x, type="l")
#par(op)

