## Integrate the logistic equation in R

library(deSolve)

## From: Soetaert, K., Cash, J. & Mazzia, F. Use R! Solving Differential equations in R. (2012). doi:10.1007/978-0-387-78171-6

# Example 1: Logistic growth:

# define the parameters:
r <- 1; K <- 10; yini <- 2

#simple differential equation is implemented in an R function called derivs that takes as arguments the current time (t), the value of the dependent variable (y) and a parameter vector (parms), and that returns the derivative, as a list.The parameters r and K, although defined outside of function derivs, are also known within the derivative function.1

derivs <- function(t, y, parms) 
list(r * y * (1-y/K))

library(deSolve) 
times <- seq(from = 0, to = 20, by = 0.2) 
out <- ode(y = yini, times = times, func = derivs, parms = NULL)



head(out, n = 3)

yini <- 12 
out2 <- ode(y = yini, times = times, func = derivs, parms = NULL)

plot(out, out2, main = "logistic growth", lwd = 2)


## Example 2 lorenz 
a <- -8/3; b <- -10; c <- 28

Lorenz <- function (t, y, parms)
 { with(as.list(y),
 {
dX <- a * X+ Y * Z 
dY <- b * (Y - Z) 
dZ <- -X * Y+ c * Y- Z 
list(c(dX, dY, dZ)) })
}

yini <- c(X = 1,Y=1,Z =1)

times <- seq(from = 0, to = 100, by = 0.01)
out <- ode(y = yini, times = times, func = Lorenz, parms = NULL)

plot(out, lwd = 2)
plot(out[,"X"], out[,"Y"], type = "l", xlab = "X", ylab = "Y", main = "butterfly")

## Example 3 Lotka volterra

alpha <- 2/3;
beta<- 4/3;
gamma <-1;
delta <-1;


LotkaVolterra <- function (t, y, parms)
 { with(as.list(y),
 {

dX <- alpha*X-beta*X*Y
dY <- delta*X*Y-gamma*Y
list(c(dX, dY)) })
}



yini <- c(X = 10,Y=10)

times <- seq(from = 0, to = 100, by = 0.01)
out <- ode(y = yini, times = times, func = LotkaVolterra, parms = NULL)

plot(out, lwd = 2)
plot(out[,"X"], out[,"Y"], type = "l", xlab = "X", ylab = "Y", main = "Lotka Volterra")

